// Demo.cpp : demonstrate how to use the libary of raid-subsystem
//

#include <iostream>

#include "arclib.h"
#include "ethernet.h"


// include the necessary interface header files
#if defined(_WIN32)
	#include "win_comm.h"
	#include "win_ioctl.h"
	#include "win_scsi.h"
#elif defined(__linux__) || defined(__FreeBSD__)
	#include "linux_comm.h"
	#include "linux_ioctl.h"
	#if defined(__linux__)
		#include "linux_scsi.h"
//
// added by hank 2006/03/07
//    
		#include "linux_pass_ioctl.h"
	#else
		#include "freebsd_scsi.h"
	#endif
#elif defined(__sun__)
	#include "solaris_comm.h"
	#include "solaris_ioctl.h"
#elif defined(__APPLE__)
	#include "darwin_userclient.h"
#endif

#ifdef _WIN32
	#pragma comment(lib, "../release/Arclib.lib")
#endif

#define MAX_CONTROLLER_SUPPORTED	4
CArclib ar[MAX_CONTROLLER_SUPPORTED];



#if defined(_WIN32)

/************************************************************************************
	Routine Name:	DiscoveryCommDevice
	Function:		Discover the controller(s) connected to the RS-232 port
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found.
	Note:			None.
************************************************************************************/
int DiscoveryCommDevice()
{
	int nTotalCtrl = 0;

	WinCommInterface *iface = new WinCommInterface();
	for ( int i = 0; i < 4 ; i++ )
	{
		if ( iface->init(i) )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
			if ( nTotalCtrl < MAX_CONTROLLER_SUPPORTED )
			{
				iface = new WinCommInterface();                
			}
		}
	}

	return nTotalCtrl;
}

/************************************************************************************
	Routine Name:	DiscoveryCardDevice
	Function:		Discover the controller(s) connected to the PCI, PCI-X, PCI-E bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryCardDevice()
{
	int nTotalCtrl = 0;

	WinIoctlInterface *iface = new WinIoctlInterface();
	for ( int i = 0; i < 16 ; i++ )
	{
		if ( iface->init(i) )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
			if ( nTotalCtrl >= MAX_CONTROLLER_SUPPORTED )
			{
				break;
			}

			iface = new WinIoctlInterface();                       
		}
	}

	return nTotalCtrl;
}

/************************************************************************************
	Routine Name:	DiscoveryInbandDevice
	Function:		Discover the controller(s) connected to the Inband scsi bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found.
	Note:			None.
************************************************************************************/
int DiscoveryInbandDevice()
{
	int nTotalCtrl = 0;

	WinScsiInterface *iface = new WinScsiInterface();
	for ( int scsi = 0; scsi <= MAX_SCSI_DEV ; scsi++ )
	{
		for ( int ch = 0; ch <= MAX_SCSI_CH ; ch++ )
		{
			for ( int target = 0; target <= MAX_SCSI_TARGETID ; target++ )
			{
				if ( iface->init(scsi, ch, target) )
				{
					ar[nTotalCtrl++].ArcInitSession(iface);
					if ( nTotalCtrl < MAX_CONTROLLER_SUPPORTED )
					{
						iface = new WinScsiInterface();
						if ( !iface )
						{
							printf("Internal Error\n");
							return -1;
						}
					}
					else
					{
						goto quit;
					}
				}
			}
		}
	}


	// scsn physical drive for our in-band device
	iface = new WinScsiInterface();
	for ( int pd = 0; pd <= MAX_SCSI_DEV ; pd++ )
	{
		if ( iface->init_physical_drive(pd) )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
			if ( nTotalCtrl < MAX_CONTROLLER_SUPPORTED )
			{
				iface = new WinScsiInterface();
				if ( !iface )
				{
					printf("Internal Error\n");
					return -1;
				}
			}
			else
			{
				goto quit;
			}
		}
	}


	quit:
	return nTotalCtrl;
}

#endif	// end of defined(_WIN32)


#if defined(__linux__)

/************************************************************************************
	Routine Name:	DiscoveryCommDevice
	Function:		Discover the controller(s) connected to the RS-232 port
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found.
	Note:			None.
************************************************************************************/
int DiscoveryCommDevice()
{
	int nTotalCtrl = 0;

	LinuxCommInterface *iface = new LinuxCommInterface();
	for ( int i = 0; i < 4 ; i++ )
	{
		if ( iface->init(i) )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
			if ( nTotalCtrl >= MAX_CONTROLLER_SUPPORTED )
			{
				break;
			}

			iface = new LinuxCommInterface();            
		}		
	}

	return nTotalCtrl;
}

/************************************************************************************
	Routine Name:	DiscoveryCardDevice
	Function:		Discover the controller(s) connected to the PCI, PCI-X, PCI-E bus
					through pass ioctl interface
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			Added by hank 2006-03-07
************************************************************************************/
int DiscoveryCardDeviceThroughPassIoctl()
{
	int nTotalCtrl = 0;
	int sg = 0;
	char stat;

	do
	{
		LinuxPassIoctlInterface  *iface = new LinuxPassIoctlInterface();

		stat = (int)iface->init(sg);
		if ( stat )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
		}
		else
		{
			delete iface;
		}
	}while ( ++sg <= 15 && nTotalCtrl < 4 );

	return nTotalCtrl;

}

/************************************************************************************
	Routine Name:	DiscoveryCardDevice
	Function:		Discover the controller(s) connected to the PCI, PCI-X, PCI-E bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryCardDevice()
{
	const char  areca_driver_name[] = "arcmsr";
	int         major_num = 0;
	int         minor_num = 0;

	char        major_num_str[4];
	char        driver_str[7];
	char        dev_name[33];
	struct stat stat_of_dev;
	int         controller_index = 0;
	mode_t      mode = 0666;

	int nTotalCtrl = 0;

	FILE *devices_file = NULL;

	devices_file = fopen("/proc/devices", "r");
	if ( !devices_file )
	{
		return 0;
	}


	// get areca's major#
	while ( fscanf(devices_file, "%3s%32s", major_num_str, dev_name) != EOF )
	{
		major_num_str[sizeof(major_num_str)-1] = 0;
		dev_name[sizeof(dev_name)-1] = 0;

		// find areca's major#
		if ( !memcmp(areca_driver_name, dev_name, 6) )
		{
			major_num = atoi(major_num_str);
			break;
		}
	}

	fclose(devices_file);

	if ( !major_num )
	{
		return 0;
	}

	mode |= S_IFCHR;

	for ( minor_num = 0; minor_num < MAX_CONTROLLER_SUPPORTED; minor_num++ )
	{
		sprintf(dev_name, "/dev/arcmsr%d", minor_num);
		if ( !stat(dev_name, &stat_of_dev) )
		{
			// device name found, check to see if the major and minor were correct.
			if ( major_num != (int)major(stat_of_dev.st_rdev) || 
				 minor_num != (int)minor(stat_of_dev.st_rdev) )
			{
				// major and minor number are not match, delete the old device name
				unlink(dev_name);
			}
			else
			{
				continue;
			}
		}

		if ( mknod(dev_name, mode, makedev(major_num, minor_num)) )
		{
			return 0;
		}
	}

	// setup areca controller(s)
	for ( controller_index = 0; controller_index < MAX_CONTROLLER_SUPPORTED; controller_index++ )
	{
		LinuxIoctlInterface *iface = new LinuxIoctlInterface();
		if ( iface )
		{
			if ( iface->init(controller_index) )
			{
				// call ArcInitSession(...) member function first before any others member functions get call
				ar[controller_index].ArcInitSession(iface);

				// save the total controller(s) we found
				nTotalCtrl++;
			}
			else
			{
				delete iface;
				iface = NULL;
			} // if
		}// if        
	}// for


	//
	// Modified by Hank 2006-03-07
	//

	// since the driver can accept two interface for client application, 
	// so if we can not find controller with primary method, try secondary.
	// 1. primary method: regular "major number" format, we first find the major number for areca raid subsystem
	//                    and make node for it, and finally open it for operation.
	// 2. secondary method: pass-through ioctl(sg interface) 
	// 
	// If primary method can not find any controller(s), try to use secondary method
	// 
	if ( nTotalCtrl == 0 )
	{
		nTotalCtrl = DiscoveryCardDeviceThroughPassIoctl();
	}

	return nTotalCtrl;

}

/************************************************************************************
	Routine Name:	DiscoveryInbandDevice
	Function:		Discover the controller(s) connected to the Inband scsi bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryInbandDevice()
{
	int    sg = 0;
	char   stat;
	int    nTotalCtrl = 0;

	do
	{
		LinuxSCSIInterface  *iface = new LinuxSCSIInterface();

		for ( int retry = 0; retry < 5; retry++ )
		{
			stat = (int)iface->init(sg);
			if ( stat )
				break;
		}

		if ( stat )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
			break;
		}
		else
		{
			delete iface;
		}
	}while ( ++sg <= 15 && nTotalCtrl < MAX_CONTROLLER_SUPPORTED );

	return nTotalCtrl;
}

#endif	// end of defined(__linux__)



#if defined (__FreeBSD__)

/************************************************************************************
	Routine Name:	DiscoveryCommDevice
	Function:		Discover the controller(s) connected to the RS-232 port
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found.
	Note:			None.
************************************************************************************/
int DiscoveryCommDevice()
{
	int nTotalCtrl = 0;

	LinuxCommInterface *iface = new LinuxCommInterface();
	for ( int i = 0; i < 4 ; i++ )
	{
		if ( iface->init(i) )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
			if ( nTotalCtrl >= MAX_CONTROLLER_SUPPORTED )
			{
				break;
			}

			iface = new LinuxCommInterface();            
		}
	}

	return nTotalCtrl;
}

/************************************************************************************
	Routine Name:	DiscoveryCardDevice
	Function:		Discover the controller(s) connected to the PCI, PCI-X, PCI-E bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryCardDevice()
{
	char buf[64];
	char rmstr[128];
	mode_t mode = 0666;
	dev_t dev;
	int     nNumofPciDevs = 0;
	int nTotalCtrl = 0;

	mode |= S_IFCHR;


	// remove all exist device node first
	for ( int i = 0; i < 4; i++ )
	{
		memset(buf, 0, sizeof(buf));
		sprintf(buf,"/dev/arcmsr%d", i);
		unlink(buf);
	}


	LinuxIoctlInterface *out = new LinuxIoctlInterface();

	for ( int mymajor = 255; mymajor > 161; mymajor-- )
	{
		for ( int myminor = 0; myminor < 4; myminor++ )
		{
			memset(buf, 0, sizeof(buf));
			sprintf(buf,"/dev/arcmsr%d", nTotalCtrl);
			dev = makedev(mymajor, myminor);
			mknod(buf, mode, dev);
			if ( out->init(nTotalCtrl) )
			{
				ar[nTotalCtrl++].ArcInitSession(out);
				out = new LinuxIoctlInterface();
			}
			else
			{
				unlink(buf);
			}
		}


		if ( nTotalCtrl )
		{
			break;
		}
	}

	return nTotalCtrl;
}



/************************************************************************************
	Routine Name:	DiscoveryInbandDevice
	Function:		Discover the controller(s) connected to the Inband scsi bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryInbandDevice()
{
	int pass = 0;
	char stat;
	int nTotalCtrl = 0;

	do
	{
		FreeBSDSCSIInterface  *iface = new FreeBSDSCSIInterface();

		for ( int retry = 0; retry < 5; retry++ )
		{
			stat = (int)iface->init(pass);
			if ( stat )
				break;
		}
		if ( stat )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
			break;
		}
		else
		{
			delete iface;
		}
	}while ( ++pass <= 15 && nTotalCtrl < MAX_CONTROLLER_SUPPORTED );


	return nTotalCtrl;
}


#endif	// end of defined (__FreeBSD__)



#if defined(__sun__)
/************************************************************************************
	Routine Name:	DiscoveryCommDevice
	Function:		Discover the controller(s) connected to the RS-232 port
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found.
	Note:			None.
************************************************************************************/
int DiscoveryCommDevice()
{
	#define LAST_DEV    4

	int nTotalCtrl = 0;

	char stat;
	SolarisCommInterface *iface = new SolarisCommInterface();
	for ( int i = 0; i < LAST_DEV ; i++ )
	{
		stat = (char)iface->init(i);
		if ( stat )
		{
			ar[nTotalCtrl++].ArcInitSession(iface);
			iface = new SolarisCommInterface();
		}
	}

	if ( iface )
	{
		delete iface;
		iface = NULL;
	}

	return nTotalCtrl;
}

/************************************************************************************
	Routine Name:	DiscoveryCardDevice
	Function:		Discover the controller(s) connected to the PCI, PCI-X, PCI-E bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryCardDevice()
{

	int nTotalCtrl = 0;
	int controller_index;

	for ( controller_index = 0; controller_index < MAX_CONTROLLER_SUPPORTED; controller_index++ )
	{
		SolarisIoctlInterface *iface = new SolarisIoctlInterface();
		if ( iface )
		{
			if ( iface->init(controller_index) )
			{
				ar[controller_index].ArcInitSession(iface);
				nTotalCtrl++;
			}
			else
			{
				delete iface;
				iface = NULL;
			} // if
		}// if
	}// for
	return nTotalCtrl;
}



/************************************************************************************
	Routine Name:	DiscoveryInbandDevice
	Function:		Discover the controller(s) connected to the Inband scsi bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryInbandDevice()
{
	// not implement yet...
	return 0;
}

#endif



#if defined(__APPLE__)
/************************************************************************************
Routine Name:	DiscoveryCommDevice
Function:		Discover the controller(s) connected to the RS-232 port
Input:			None.
Output:			None.
Return Values:	1. Total controller(s) found.
Note:			None.
************************************************************************************/
int DiscoveryCommDevice()
{
	// not implement yet...
	return 0;
}

/************************************************************************************
	Routine Name:	DiscoveryCardDevice
	Function:		Discover the controller(s) connected to the PCI, PCI-X, PCI-E bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryCardDevice()
{
	int nTotalCtrl = 0;
	int controller_index;

	for ( controller_index = 0; controller_index < MAX_CONTROLLER_SUPPORTED; controller_index++ )
	{
		DarwinUserClientInterface *iface = new DarwinUserClientInterface();

		if ( iface->init(controller_index) )
		{
			ar[controller_index].ArcInitSession(iface);
			nTotalCtrl++;
		}
		else
		{
			delete iface;
			iface = NULL;
		}
	}
	return nTotalCtrl;
}



/************************************************************************************
	Routine Name:	DiscoveryInbandDevice
	Function:		Discover the controller(s) connected to the Inband scsi bus
	Input:			None.
	Output:			None.
	Return Values:	1. Total controller(s) found
	Note:			None.
************************************************************************************/
int DiscoveryInbandDevice()
{
	// not implement yet...
	return 0;
}

#endif


/*****************************************************************
Function: Sub-routine of "Hardware Monitor Information" command
Input:	1.argc
		2.argv
Output:	1. ARC_STATUS
*****************************************************************/

ARC_STATUS Exe_Hw_Info( CArclib &ctrl )
{
	ARC_STATUS  stat;
	sSYSTEM_INFO    sysInfo;
	sHwMonInfo  voltageInfo, fanInfo, hddTempInfo, batteryInfo;
	sHwMon5010  hwInfo5010;
	int     i;
	bool    got5010 = false;
	bool    gotRaidCard = false;

	char strId[64];
	char modelStr[9];

	memset(strId, 0, sizeof(strId));
	memset(modelStr, 0, sizeof(modelStr));

	stat = ctrl.ArcGetIdentifyString(strId);
	if ( stat != ARC_SUCCESS )
	{
		return stat;
	}

	memcpy(modelStr, (const void*)&strId[20], 8);


	if ( strcmp(modelStr, "ARC-5010")==0 || strcmp(modelStr, "ARC-6010")==0 )
	{
		got5010 = true;
	}

	if ( strncmp(modelStr, "ARC-11", 6) == 0 || strncmp(modelStr, "ARC-12", 6)==0 )
	{
		gotRaidCard = true;
	}

	if ( got5010 )
	{
		stat = ctrl.ArcGetHWMon5010(&hwInfo5010);
		if ( stat != ARC_SUCCESS )
		{
			return stat;
		}
	}
	else
	{
		stat = ctrl.ArcGetSysInfo(&sysInfo);
		if ( stat != ARC_SUCCESS )
		{
			return stat;
		}

		stat = ctrl.ArcGetHWMon(HWMON_FAN_INFO, &fanInfo);
		if ( stat != ARC_SUCCESS )
		{
			return stat;
		}

		stat = ctrl.ArcGetHWMon(HWMON_TMP_INFO, &hddTempInfo);
		if ( stat != ARC_SUCCESS )
		{
			return stat;
		}

		stat = ctrl.ArcGetHWMon(HWMON_VOL_INFO, &voltageInfo);
		if ( stat != ARC_SUCCESS )
		{
			return stat;
		}

		stat = ctrl.ArcGetHWMon(HWMON_BATTERY_INFO, &batteryInfo);
		if ( stat != ARC_SUCCESS )
		{
			return stat;
		}

	}

	printf("The Hardware Monitor Information\n");
	printf("===========================================\n");
	if ( got5010 )	  // 5 bays ARC-5010 or ARC-6010
	{
		printf("Controller Temp : %d\n", hwInfo5010.monCtrlTemp);
		printf("BackPlane Temp  : %d\n", hwInfo5010.monBkPlaneTemp);
		printf("Fan Speed (RPM) : %d\n", hwInfo5010.monFan);
		printf("Power +12.00V   : %2.3lf\n", (double)hwInfo5010.mon12V/(double)1000.0);
		printf("Power +5.00V    : %2.3lf\n", (double)hwInfo5010.mon5V/(double)1000.0);
		printf("Power +3.30V    : %2.3lf\n", (double)hwInfo5010.mon3_3V/(double)1000.0);
		printf("Power +1.50V    : %2.3lf\n", (double)hwInfo5010.mon1_5V/(double)1000.0);
	}
	else if ( gotRaidCard )	   // RaidCard
	{
		for ( i = 0; i < fanInfo.HwItemCount; i++ )
		{
			if ( fanInfo.ItemInfo[i] == 0xffff )
			{
				printf("Fan#%d Speed (RPM)  : N.A.\n", i+1);
			}
			else
			{
				printf("Fan#%d Speed (RPM)  : %d\n", i+1, fanInfo.ItemInfo[i]);
			}
		}

		for ( i = 0; i < batteryInfo.HwItemCount; i++)
		{
			if(batteryInfo.ItemInfo[i] == BBM_NOT_INSTALLED)
			{
				printf("Battery Status      : Not Installed\n");
			}
			else if(batteryInfo.ItemInfo[i] == BBM_FAILED)
			{
				printf("Battery Status      : Failed\n");
			}
			else
			{
				printf("Battery Status      : %d%%\n", batteryInfo.ItemInfo[i]);
			}
		}

		for ( i = 0; i < hddTempInfo.HwItemCount; i++ )
		{
			printf("HDD #%-2d Temp.       : %d\n", i+1, hddTempInfo.ItemInfo[i]);
		}       
	}
	else // subsystem
	{
		for ( i = 0; i < voltageInfo.HwItemCount; i+=2 )
		{
			printf("Power +%-5.2lfV       : %2.3lf\n", (double)voltageInfo.ItemInfo[i]/(double)1000.0, 
				   (double)voltageInfo.ItemInfo[i+1]/(double)1000.0);
		}

		for ( i = 0; i < fanInfo.HwItemCount; i++ )
		{
			if ( fanInfo.ItemInfo[i] == 0xffff )
			{
				printf("Fan#%d Speed (RPM)  : N.A.\n", i+1);
			}
			else
			{
				printf("Fan#%d Speed (RPM)  : %d\n", i+1, fanInfo.ItemInfo[i]);
			}
		}

		// show controller temperature first
		printf("Controller Temp.    : %d\n", hddTempInfo.ItemInfo[0]);
		//printf("Battery Status     : Not Installed\n");
		for ( i = 1; i < hddTempInfo.HwItemCount; i++ )
		{
			printf("HDD #%-2d Temp.       : %d\n", i, hddTempInfo.ItemInfo[i]);
		}       

	}
	printf("===========================================\n");

	return ARC_SUCCESS;
}


/*****************************************************************
Function: Show the event(s) of raid-subsystem
Input:
Output:	1. ARC_STATUS
*****************************************************************/
ARC_STATUS Exe_Event_Info( CArclib &ctrl )
{
	ARC_STATUS  stat = ARC_SUCCESS;
	char        ptr1[64];                   
	char        tmpStr[64];
	int         nCount = 0;				// total number of event objects we got	
	int         index;  
	char        timeBuf[64];
	int         nEventFlag = FLAG_ALL;	// the flag will be passed into ArcGetReqEventPage(...)
	sSYSTEM_INFO    sysInfo;

	//the time event occurred, defined in arclib.h
	pSYS_TIME evtTime = (pSYS_TIME)new sSYS_TIME;
	// event object defined in arclib.h
	pEVENT_DATA pEventObject = (pEVENT_DATA)new sEVENT_DATA;    

	*((LONG *)evtTime) = 0L;
	memset(pEventObject, 0, sizeof(sEVENT_DATA));

	stat = ctrl.ArcGetSysInfo(&sysInfo);
	if ( stat != ARC_SUCCESS )
	{
		goto quit;
	}

	/* 
		We should call ArcSetTime() to correct the raid-subsystem time
	*/
	stat = ctrl.ArcSetTime();
	if ( stat != ARC_SUCCESS )
	{
		goto quit;
	}
	/* 
		Call ArcGetReqEventPage() to get the required event page from raid subsystem.
		There are 4 event pages reside in the raid subsystem and each contains 16 event objects.			
	*/
	stat = ctrl.ArcGetReqEventPage(nEventFlag, &nCount);
	if ( stat != ARC_SUCCESS )
	{
		goto quit;
	}
	if ( stat == ARC_SUCCESS )
	{
		if ( nCount != 0 )
		{
			printf("Date-Time               Device              Event Type	       \n");
			printf("================================================================================\n");
			for ( index = 0; index <= nCount-1; index++ )
			{
				/*
					Call ArcGetEventObject function to extract the individual event object from event page
				*/
				if ( ctrl.ArcGetEventObject( index, pEventObject ) )
				{
					memset(timeBuf, 0, sizeof(timeBuf));
					// If tmFlag == 1 ==> use YY/MM/DD/HH/MM/SS else use time tick
					*((LONG *)evtTime) = pEventObject->evtTime;
					if ( *((LONG *)evtTime) & 0x80000000 )
					{
						sprintf(timeBuf, "%d-%d-%d  %d:%d:%d", 
								evtTime->u.tmYear + 2000, 
								evtTime->u.tmMonth,
								evtTime->u.tmDate,
								evtTime->u.tmHour,
								evtTime->u.tmMinute,
								evtTime->u.tmSecond);
					}
					else
					{
						sprintf(timeBuf, "%d", evtTime->x.tmTick );
					}

					memset( ptr1, 0, sizeof(ptr1));
					memset( tmpStr, 0, sizeof(tmpStr));

					/* 
						Parse the event object we got
					*/
					switch ( (int)pEventObject->evtCategory )
					{
					/* 
						The member "evtStr" of event object structure contains the string to indicate
						which raidset owns this event object
					*/
					case EVENT_RAIDSET:		// raidset event							
						strcpy( ptr1, htmRaidEvent[pEventObject->evtType]); 
						memcpy(tmpStr, pEventObject->evtStr, 16);
						printf("%-24s", timeBuf);
						printf("%-20s%s\n", tmpStr, ptr1);                          
						break;

						/* 
							The member "evtStr" of event object structure contains the string to indicate
							which volumeset owns this event object
						*/
					case EVENT_VOLUMESET: // volumeset event
						strcpy( ptr1, htmVolEvent[pEventObject->evtType]);  
						memcpy(tmpStr, pEventObject->evtStr, 16);
						printf("%-24s", timeBuf);
						printf("%-20s%s\n", tmpStr, ptr1);
						break;

					case EVENT_DEVICE:	// device event
						sprintf(tmpStr, "IDE Channel #%2d", pEventObject->evtChannel + 1);
						printf("%-24s", timeBuf);
						printf("%-20s%s\n", tmpStr, htmDevEvent[pEventObject->evtType]);    
						break;

					case EVENT_HOST:	// host event
						/* this is Host SCSI events */
						printf("%-24s", timeBuf);
						if ( sysInfo.gsiScsiHostChannels )
						{
							printf("%-20s%s\n",
								   eventCat[pEventObject->evtCategory], 
								   htmScsiHostEvent[pEventObject->evtType]);
						}
						else
						{
							printf("%-20s%s\n",
								   eventCat[pEventObject->evtCategory], 
								   htmIdeHostEvent[pEventObject->evtType]);
						}
						break;

					case EVENT_HW_MONITOR:	// hardware monitor event
						printf("%-24s", timeBuf);
						printf("%-20s%s\n",
							   eventCat[pEventObject->evtCategory], 
							   htmHwMonEvent[pEventObject->evtType]);
						break;
					case EVENT_NEW_83782D:
						memcpy(tmpStr, pEventObject->evtStr, 16);
						printf("%-24s", timeBuf);
						printf("%-20s%s\n", tmpStr, htmNewEventStr[pEventObject->evtType]);
						break;

					case EVENT_NO_EVENT:
//							printf("		No Event\n");	
					default:
						break;  

					}// end switch
				}// end if
			}// end for
			printf("================================================================================\n");
		}//end if
	}// end if

	quit:
	if ( pEventObject )
		delete pEventObject;
	if ( evtTime )
		delete evtTime;

	return stat;
}

int DemoForEthernetInterface()
{
	sSYSTEM_INFO    sysInfo;		// see arclib.h for details
	ARC_STATUS      stat;			// see arclib.h for details

	/* For Ethernet Interface */
	// allocate an instance of the "Interface" and initialize it with a proper IP address
	EthernetInterface *ei = new EthernetInterface();
	if ( !ei->init("192.168.0.50") )
	{
		printf("Failed to init interface\n");
		return 1;
	}

	// assign the interface to the library
	stat = ar[0].ArcInitSession(ei);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcInitSession(...), Error Code = %X.\n", stat);
		return 1;
	}

	// get the raid-subsystem information
	stat = ar[0].ArcGetSysInfo(&sysInfo);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcGetSysInfo(...), Error Code = %X.\n", stat);
		return 1;
	}


	// print out the events of raid-subsystem
	Exe_Event_Info(ar[0]);

	// create a raidset with drives 1~8 ( 11111111b )
	stat = ar[0].ArcChkPassword("0000");   
	stat = ar[0].ArcCreateRaidSet(0x000000FF, "RaidSet#1");
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateRaidSet(...), Error Code = %X.\n", stat);
		return 1;
	}

	sVolumeSetCap   volCap;
	memset(&volCap, 0, sizeof(sVolumeSetCap));
	volCap.structs_size = sizeof(sVolumeSetCap);
	volCap.raid = 0;	// first raidset
	strcpy((char *)volCap.volname, "VolumeSet#1");
	volCap.capacity = 10.0;
	volCap.raidlevel = VOLUME_TYPE_RAID5;
	volCap.stripsize = STRIPE_SIZE_64K;
	volCap.channel = 0;
	volCap.id = 0;
	volCap.lun = 0;
	volCap.tag = ENABLE;
	volCap.cache = ENABLE;
	volCap.speed = SCSI_SPEED_160;
	volCap.quickinit = FOREGROUND_INIT;

	stat = ar[0].ArcCreateVolSet(&volCap);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateVolSet(...), Error Code = %X.\n", stat);
		return 1;
	}

	/* IMPORTANT: DO NOT delete the instance of the "Interface" by yourself,
	   library will take care of that */

	system("pause");
	return 0;
}

int DemoForRS232Interface()
{
	sSYSTEM_INFO    sysInfo;		// see arclib.h for details
	ARC_STATUS      stat;			// see arclib.h for details
	int             total_ctrl = 0;

	/* For RS232 Interface */
	// allocate an instance of the "Interface" and initialize it with "COM1"

	total_ctrl = DiscoveryCommDevice();
	if ( total_ctrl == 0 )
	{
		printf("No controller(s) found!!\n");
		return 1;
	}

	// get the raid-subsystem information
	stat = ar[0].ArcGetSysInfo(&sysInfo);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcGetSysInfo(...), Error Code = %X.\n", stat);
		return 1;
	}


	// print out the events of raid-subsystem
	Exe_Event_Info(ar[0]);


	// create a raidset with drives 1~8 ( 11111111b )
	stat = ar[0].ArcChkPassword("0000");   
	stat = ar[0].ArcCreateRaidSet(0x000000FF, "RaidSet#1");
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateRaidSet(...), Error Code = %X.\n", stat);
		return 1;
	}

	sVolumeSetCap   volCap;
	memset(&volCap, 0, sizeof(sVolumeSetCap));
	volCap.structs_size = sizeof(sVolumeSetCap);
	volCap.raid = 0;	 // create this volume in the first RaidSet
	strcpy((char *)volCap.volname, "VolumeSet#1");
	volCap.capacity = 10.0;
	volCap.raidlevel = VOLUME_TYPE_RAID5;
	volCap.stripsize = STRIPE_SIZE_64K;
	volCap.channel = 0;
	volCap.id = 0;
	volCap.lun = 0;
	volCap.tag = ENABLE;
	volCap.cache = ENABLE;
	volCap.speed = SCSI_SPEED_160;
	volCap.quickinit = FOREGROUND_INIT;

	stat = ar[0].ArcCreateVolSet(&volCap);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateVolSet(...), Error Code = %X.\n", stat);
		return 1;
	}


	/* IMPORTANT: DO NOT delete the instance of the "Interface" by yourself,
	   library will take care of that */

	system("pause");
	return 0;
}



int DemoForIoctlInterface()
{
	sSYSTEM_INFO    sysInfo;		// see arclib.h for details
	ARC_STATUS      stat;			// see arclib.h for details
	int             total_ctrl = 0;

	/* For Ioctl Interface */
	// Find the controller we supported in the system up to MAX_CONTROLLER_SUPPORTED(4)
	total_ctrl = DiscoveryCardDevice();
	if ( total_ctrl == 0 )
	{
		printf("No controller(s) found!!\n");
		return 1;
	}
	printf("%d controller(s) found.\n", total_ctrl);

	// get the raid-subsystem information for controller#0
	stat = ar[0].ArcGetSysInfo(&sysInfo);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcGetSysInfo(...), Error Code = %X.\n", stat);
		return 1;
	}


	// print out the events of raid-subsystem
	Exe_Event_Info(ar[0]);


	// create a raidset with drives 1~4 ( 00001111b )
	stat = ar[0].ArcChkPassword("0000");    
	stat = ar[0].ArcCreateRaidSet(0x0000000F, "RaidSet#1");
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateRaidSet(...), Error Code = %X.\n", stat);
		return 1;
	}

	sVolumeSetCap   volCap;
	memset(&volCap, 0, sizeof(sVolumeSetCap));
	volCap.structs_size = sizeof(sVolumeSetCap);
	volCap.raid = 0;	// create this volume in the first RaidSet
	strcpy((char *)volCap.volname, "VolumeSet#1");
	volCap.capacity = 10.0;
	volCap.raidlevel = VOLUME_TYPE_STRIPING;
	volCap.stripsize = STRIPE_SIZE_64K;
	volCap.channel = 0;
	volCap.id = 0;
	volCap.lun = 0;
	volCap.tag = ENABLE;
	volCap.cache = ENABLE;
	volCap.speed = SCSI_SPEED_160;
	volCap.quickinit = FOREGROUND_INIT;

	stat = ar[0].ArcCreateVolSet(&volCap);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateVolSet(...), Error Code = %X.\n", stat);
		return 1;
	}


	/* IMPORTANT: DO NOT delete the instance of the "Interface" by yourself,
	   library will take care of that */

	system("pause");
	return 0;
}

int DemoForInbandInterface()
{
	sSYSTEM_INFO    sysInfo;		// see arclib.h for details
	ARC_STATUS      stat;			// see arclib.h for details
	int             total_ctrl = 0;

	/* For Inband Interface */
	// Find the controller we supported in the system up to MAX_CONTROLLER_SUPPORTED(4)
	total_ctrl = DiscoveryInbandDevice();
	if ( total_ctrl == 0 )
	{
		printf("No controller(s) found!!\n");
		return 1;
	}
	printf("%d controller(s) found.\n", total_ctrl);

	// get the raid-subsystem information for controller#0
	stat = ar[0].ArcGetSysInfo(&sysInfo);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcGetSysInfo(...), Error Code = %X.\n", stat);
		return 1;
	}


	// print out the events of raid-subsystem
	Exe_Event_Info(ar[0]);


	// create a raidset with drives 1~4 ( 00001111b )
	stat = ar[0].ArcChkPassword("0000");    
	stat = ar[0].ArcCreateRaidSet(0x0000000F, "RaidSet#1");
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateRaidSet(...), Error Code = %X.\n", stat);
		return 1;
	}

	sVolumeSetCap   volCap;
	memset(&volCap, 0, sizeof(sVolumeSetCap));
	volCap.structs_size = sizeof(sVolumeSetCap);
	volCap.raid = 0;	// create this volume in the first RaidSet
	strcpy((char *)volCap.volname, "VolumeSet#1");
	volCap.capacity = 10.0;
	volCap.raidlevel = VOLUME_TYPE_STRIPING;
	volCap.stripsize = STRIPE_SIZE_64K;
	volCap.channel = 0;
	volCap.id = 0;
	volCap.lun = 0;
	volCap.tag = ENABLE;
	volCap.cache = ENABLE;
	volCap.speed = SCSI_SPEED_160;
	volCap.quickinit = FOREGROUND_INIT;

	stat = ar[0].ArcCreateVolSet(&volCap);
	if ( stat != ARC_SUCCESS )
	{
		printf("Failed to ArcCreateVolSet(...), Error Code = %X.\n", stat);
		return 1;
	}


	/* IMPORTANT: DO NOT delete the instance of the "Interface" by yourself,
	   library will take care of that */

	system("pause");
	return 0;   

}


int main(int argc, char* argv[])
{
	//
	// choice an interface you want to implement, and comment the others
	// 
	DemoForEthernetInterface();
	DemoForRS232Interface();
	DemoForIoctlInterface();
	DemoForInbandInterface();
	return 0;
}


